#!/usr/bin/perl -wT
# This script written with UTF-8N. for Perl 5.8x later.
# 2010/07/16 ver0.01 [-TEST-] 記事データをテンプレートに代入して表示。
# 2010/07/16 ver0.02 [-TEST-] 記事内のURLとメールアドレスの拾い上げを実装。
# 2010/07/16 ver0.03 [-TEST-] 前に戻る、ウィンドウを閉じる、非表示の実装。
# 2010/07/16 ver0.04 [-TEST-] 年月日(YYYY-MM-DD)を、和暦YYYY年MM月DD日にすり替え。
# 2010/07/27 ver0.05 [STABLE] 共有ルーチンを外部に待避。
# 2010/07/27 ver1.00 [STABLE] リリース版。
# 2010/07/29 ver1.01 [-FIX.-] $id から、数字以外を削除する様にサニタイズ処理を追加。
# 2010/08/17 ver1.02 [UPDATE] 本文を一段落ごとに<p>タグで括るように変更。
# 2010/09/21 ver1.10 [UpDate] 新着ニュースのフラグ対応。
# 2011/01/27 ver1.11 [UpDate] SQL管理情報を分離する。
# 2011/01/27 ver1.12 [UpDate] 設定されたカテゴリーの記事を読む処理を追加。
# 2011/01/28 ver1.13 [UpDate] 記事のカテゴリに対応したSQLの実テーブル名を取得する処理を追加し、それを分離する。
# 2011/09/02 ver1.84 [-TEST-] DBに設定したユーザー認証をチェックするテスト。
# 2011/09/07 ver1.85 [-TEST-] 各ニュース出力に使用期限を設定して制限するテスト。
# 2011/09/14 ver1.86 [UpDate] 要求に応じたニュースの原稿のみを出力できるように改廃。
# 2011/09/21 ver1.90 [PreREL] 出力データに対してURIエンコードする処理を追加。
# 2011/10/24 ver1.95 [PreREL] 正式版テストリリース。
# 2011/10/26 ver1.96 [PreREL] 正規表現オプションを最適化（ｉ= 大小文字不問、o = ループ処理内での正規表現コンパイルを１度に制限）
# 2011/10/26 ver1.97 [PreREL] li 内に div を配置する特殊表示仕様(非正規)を追加。
# 2011/10/26 ver1.98 [PreREL] 記事見出し、バックナンバーの年月日表示をクライアント側で置換処理できるように変更。
# 2011/10/27 ver1.99 [PreREL] バックナンバー時にＡタグの内容を正しく書き分けるように調整。
# 2011/10/27 ver2.00 [STABLE] 正式リリース版。
# 2012/07/12 ver2.10 [UPDATE] 未来の日付になっている記事をスキップ（非表示）にする処理を追加。
# 2012/07/12 ver2.20 [UPDATE] 見出し一覧リストのリンクについて _self か _blank を選べるように変更。
# 2012/07/13 ver3.00 [STABLE] リリース版。ＵＲＬ拾い上げの正規表現を完全版に差し替え。
# 2012/07/25 ver3.10 [-FIX.-] ゼイタックスＨＰで、見出し記事リストが id の降順にならなかったのを SQL で修正。
# by TAXCOM co.,ltd.
#########################################################################

use strict;
use warnings;
#use CGI;
#use CGI::Carp qw(fatalsToBrowser);
use Encode;
#use Encode::Guess;
use DBI();

use Time::Local; # 月末日の算出で使用する。

#use LWP::Simple; # httpによるページを取得する

# ハードコードされたテキストを内部表現で扱う。
use utf8;

#binmode STDOUT, ':encoding(UTF-8)';
#binmode STDOUT, ':encoding(EUC-JP)';

################################################################

	# 初期化（セキュリティ確保）処理
	$ENV{'PATH'} = "/bin:/usr/bin";# 最低限のPATHだけに初期化
	delete @ENV{'IFS', 'CDPATH', 'ENV', 'BASH_ENV'};# 不要な環境変数を削除

	# CGI.pm を使用する場合には下記を設定する。
#	$CGI::DISABLE_UPLOADS = 1; # 1 で、データのアップロード機能を無効に
#	$CGI::POST_MAX = 0; # POSTデータのサイズ上限を0バイトに

################################################################

{ # main

	# 共有ルーチンフォルダのパス情報
	my $perlib_path = './perlib/';

	# 共有ルーチンの読み込み
	require $perlib_path . 'get_submit_data.pl';
	require $perlib_path . 'get_form_data.pl';
	require $perlib_path . 'esqueuel_ctl.cgi';
	require $perlib_path . 'get_news_table_name.pl';
	require $perlib_path . 'uri_decode.pl';
	require $perlib_path . 'uri_encode.pl';
	require $perlib_path . 'tag_escape.pl';
	require $perlib_path . 'crlf2br.pl';

	require $perlib_path . 'gen_ymdhms.pl';
	require $perlib_path . 'ymd14substr.pl';
	require $perlib_path . 'GetLastDay.pl';

	# 記事の見出しリストのデフォルト行数
		# 既定値：6
	my $default_subject_line = 6;

	# 記事の見出しリストの最大行数
		# 初期値：12 … あまり増やすとサーバー負荷が苛烈にアップします。
	my $subject_line_max = 12;

	# 記事の見出しリストの最小行数
		# 既定値：1
	my $subject_line_min = 1;

	# バックナンバーの最過去の年代をセットする。
		# ここの年代よりも過去にはさかのぼれなくなる。
		# 現状では2011年より過去の記事がストックされていないので、2011をセット。
		my $backnum_axis = 2011;

	# バックナンバーを何年前まで参照可能にするか？
		# ex. 0 → 基準となる年(=今年)のみ選択可能にする。
		# ex. 1 → 基準とした年と、その前年が選択できる。
	my $retroactive = 3; 

	# プログラムのルートディレクトリ
	# (環境変数が使えない時は絶対パス指定で)
	my $base_path = '';
		#my $base_path = $ENV{'SCRIPT_FILENAME'};
		# ディレクトリ情報以降を削除する
		#$base_path =~ s!^(/.*/).*$!$1!;

	# 記事本文を閲覧するためのCGIである view_news.cgi までのパス情報
	my $client_path = '###READER_PATH###';

	# 記事一覧リストの書式
#	my $list_prefix = '<ul>';
#		my $list_content_prefix = '<li>';
#		my $list_content_suffix = '</li>';
#	my $list_suffix = '</ul>';
	my $list_prefix;
		my $list_content_prefix;
		my $list_content_suffix;
	my $list_suffix;

	# GMTからの時差情報をセットする
	# 日本国内の時差の場合は、JST-9 をセットする。
	# JST = 日本標準時
	# GMT = グリニッジ平均時
	my $gmt_time_difference = 'JST-9';

	# サーバーの現在時刻を取得する。
	my ($jst9_year, $jst9_month, $jst9_day, $jst9_hour, $jst9_minute, $jst9_second) = &gen_ymdhms(\$gmt_time_difference);

	# 現在のサーバー時刻を unix_timestamp で取得する。
	my $just_now = time();

	# ニュース記事からＵＲＬをピックアップする公式
	my $uri_match = q{\b(?:https?|shttp)://(?:(?:[-_.!~*'()a-zA-Z0-9;:&=+$,]|%[0-9A-Fa-f} . q{][0-9A-Fa-f])*@)?(?:(?:[a-zA-Z0-9](?:[-a-zA-Z0-9]*[a-zA-Z0-9])?\.)} . q{*[a-zA-Z](?:[-a-zA-Z0-9]*[a-zA-Z0-9])?\.?|[0-9]+\.[0-9]+\.[0-9]+\.} . q{[0-9]+)(?::[0-9]*)?(?:/(?:[-_.!~*'()a-zA-Z0-9:@&=+$,]|%[0-9A-Fa-f]} . q{[0-9A-Fa-f])*(?:;(?:[-_.!~*'()a-zA-Z0-9:@&=+$,]|%[0-9A-Fa-f][0-9A-} . q{Fa-f])*)*(?:/(?:[-_.!~*'()a-zA-Z0-9:@&=+$,]|%[0-9A-Fa-f][0-9A-Fa-f} . q{])*(?:;(?:[-_.!~*'()a-zA-Z0-9:@&=+$,]|%[0-9A-Fa-f][0-9A-Fa-f])*)*)} . q{*)?(?:\?(?:[-_.!~*'()a-zA-Z0-9;/?:@&=+$,]|%[0-9A-Fa-f][0-9A-Fa-f])} . q{*)?(?:#(?:[-_.!~*'()a-zA-Z0-9;/?:@&=+$,]|%[0-9A-Fa-f][0-9A-Fa-f])*} . q{)?};

	# ＵＲＬリンクをクリックした時の挙動制御
	my $uri_target = '_blank';

	# 新着記事を表すマーキング
		# 色づけは、スタイルシートで行えます。
		# ＣＳＳにある .new_flag の値で文字飾りや見出しとの距離を指定してください。
	my $new_mark = ' NEW';

	# ニュース記事からメールアドレスをピックアップする公式
	my $mail_match = '[-a-z0-9._]+\@[-a-z0-9._]+\.([a-z]{2,3})';

	# 投稿されたニュースデータを受け取る。
	# 内部表現(フラグ付きUTF8)への変換処理(decode)されます。
	my $submit_data = &get_submit_data;
	my $hash_ref = &get_form_data($submit_data);
	my %form = %$hash_ref;

	my $id_src = $form{'id_src'};		# 要求されたユーザーID

	# IDが無ければ、about:blank にすっ飛ばす。
	if($id_src eq ''){
#		print "Location: about:blank\n\n";
		print "Status: 301 Moved Permanently\n";
		print "Location: http://www.taxcom.co.jp/\n\n";

		exit;
	};

	# バックナンバーのリストスタイル定義
	my $list_style = $form{'list_style'};

	# 記事の日付フォーマットの設定
		# NONE：日付を表示しない。
		# Y4MD：yyyy年mm月dd日で表示
		# Y2MD：yy年mm月dd日で表示
		# MD：月日を表示
		# D：日だけを表示
			# 注意：直下にある「日付表示の区切り子」も併せて設定してください。
			# バックナンバーのＣＳＳの調整も行ってください。
			# .info_cage の margin-left の em値 を増減すると、日付と記事タイトルの間隔を調整できます。
	my $date_form = $form{'date_form'};

#######################################################################################
# ユーザー認証を処理するテスト

	print "Content-type: text/html\n\n";

	my $id_src = $form{'id_src'};		# 要求されたユーザーID
	my $cat_src = $form{'cat_src'};		# 要求されたニュース・カテゴリー
	my $news_src = $form{'news_src'};	# 要求された記事No.
	my $news_order =  $form{'news_order'}; # 要求された記事リストの本数

	my $out_encode_type = $form{'enc'};

	# ゼイタックス出力用にするかどうかの判断フラグ
	my $time_stamp = $form{'time_stamp'};

	# 表示するバックナンバーの年月を取得する。
	my $backnum_year = $form{'backnum_year'};
	my $backnum_month = $form{'backnum_month'};
	my $list_style = $form{'list_style'};

	# サニタイズ処理。
	$id_src =~ s/[^0-9a-f]//ig;

	# 数字に b が含まれていたらバックナンバーを表示する。
		# 数字に c が含まれていたら見出し一覧を表示する。
		# 数字だけならば 記事No. として扱う。
	$news_src =~ s/[^0-9bc]//ig;

	# 先頭から40文字だけを切り出し。
	# 単純抽出は、正規表現よりも substr 等の方が早いらしい。
	$id_src = substr($id_src, 0, 40);

	# 処理するニュースカテゴリーをチェックし、SQL上の実テーブル名を取得する。
	my ($news_category_table, $news_category_name) = &get_news_table_name($cat_src);


# IDのマッチングを行う

	my $auth_table = 'waitress';

	my($db_user, $db_pass, $db_type, $db_name, $db_locate, $db_port, $db_attr_ref) = &sqlctrl();
	my $dbh = DBI->connect("dbi:$db_type:$db_name:$db_locate:$db_port", $db_user, $db_pass, $db_attr_ref) or die $DBI::errstr;

	# ユーザーIDと、各サービスの有効期限を unix_timestamp (1970-01-01からの積算秒数)に変換して取得する。
		# DATETIME値の場合、1970-01-01 09:00:00 が、ゼロ秒に相当する。…JST-9(+9hours) だから、９時が起点？
	my $sth = $dbh->prepare("SELECT passphrase, unix_timestamp(zeimu_news)
											  , unix_timestamp(keiei_news)
											  , unix_timestamp(kokuzei_news)
											  , unix_timestamp(kokuzei_tsutatsu)
											  , unix_timestamp(zaimu_news)
							 FROM $auth_table WHERE passphrase=?") or die $DBI::errstr;

	$sth->bind_param( 1, $id_src ); #テーブルの１列目(カラム)

	$sth->execute() or die $DBI::errstr;

	# DBから取得したレコードの列データ(２次元テーブル)と変数を関連づける。
	my ($out_id, $lease_zeimu_news, $lease_keiei_news, $lease_kokuzei_news, $lease_kokuzei_tsutatsu, $lease_zaimu_news);
	$sth->bind_columns(\$out_id, \$lease_zeimu_news, \$lease_keiei_news, \$lease_kokuzei_news, \$lease_kokuzei_tsutatsu, \$lease_zaimu_news);

	$sth->fetch;

	# ユーザーIDの有無をチェック有れば処理。なければ終了する。
	if($out_id){

		# 出力用の変数。
		my $out_data;

		# バックナンバーリストの書式設定オプションを１文字分抜き出す。
		$list_style = substr($list_style, 0, 2);

		# バックナンバーリストのフォーマット指定を反映する
		# 未定義の場合、あるいは P か p の場合には、P タグを適用する
		if($list_style =~m/^ud$/i){
			$list_prefix = '<ul class="ul_prefix">';
				$list_content_prefix = '<li class="li_prefix">';
				$list_content_suffix = '</li>';
			$list_suffix = '</ul>';

		}elsif($list_style =~m/d/i){
			$list_prefix = '';
				$list_content_prefix = '';
				$list_content_suffix = '';
			$list_suffix = '';

		}elsif($list_style =~m/u/i){
			$list_prefix = '<ul class="ul_prefix">';
				$list_content_prefix = '<li class="li_prefix">';
				$list_content_suffix = '</li>';
			$list_suffix = '</ul>';

		}elsif($list_style =~m/o/i){
			$list_prefix = '<ol class="ol_prefix">';
				$list_content_prefix = '<li  class="li_prefix">';
				$list_content_suffix = '</li>';
			$list_suffix = '</ol>';

		}else{
			$list_prefix = '';
				$list_content_prefix = '<p class="p_prefix">';
				$list_content_suffix = '</p>';
			$list_suffix = '';

		}


# ##########################################################################################
# ▼ 要求されたカテゴリの年月から、バックナンバーを出力する処理
# ##########################################################################################

		if($news_src =~ m/[b]/i){

			# 日付フォーマットが未定義なら日付のみ表示にセット
			if($date_form eq ''){$date_form = 'D'};

			# 表示するバックナンバーの年月から、数字以外を削除する。
			$backnum_year =~ s/[^0-9]//g;
			$backnum_month =~ s/[^0-9]//g;

			# 日数が一桁の場合は、十の位にゼロを補う
			if($backnum_month < 10){$backnum_month = "0" . $backnum_month;}

			# 表示するバックナンバーの年数が４桁の数字、月数が２桁の数字以外なら、それぞれ現在の年月を代入する。
			if($backnum_year !~ /^\d{4}$/){$backnum_year = $jst9_year; };
			if($backnum_month !~ /^\d{2}$/){$backnum_month = $jst9_month; };

			# バックナンバー取得の月初と月末の日付を作成。
			my $start_ymd = $backnum_year . "-" . $backnum_month . "-" . "01";

			# 月末日を算出。
			my $last_day = &GetLastDay($backnum_year, $backnum_month);
			my $end_ymd = $backnum_year . "-" . $backnum_month . "-" . $last_day;

			my $sth;

 			# ゼイタックスHP向けの日付を要求されていたら、日付の範囲指定を切り替える
			if($time_stamp eq 'zeitax'){

				$sth = $dbh->prepare("SELECT * FROM $news_category_table WHERE zeitax_publish BETWEEN ? AND ? ORDER BY zeitax_publish DESC") or die $DBI::errstr;

			}else{

				$sth = $dbh->prepare("SELECT * FROM $news_category_table WHERE publish BETWEEN ? AND ? ORDER BY publish DESC") or die $DBI::errstr;

			}

			#プレースホルダーで、パラメーターをセット。
			$sth->bind_param( 1, $start_ymd ); # バックナンバーの月初日(YYYY-MM-DD)
			$sth->bind_param( 2, $end_ymd ); # バックナンバーの月末日(YYYY-MM-DD)

			$sth->execute() or die $DBI::errstr;

			# DBから取得したレコードの列データ(２次元テーブル)と変数を関連づける。
			my ($out_id, $out_ymd, $out_news_flag, $out_title, $out_body, $out_zeitax_publish);
			$sth->bind_columns(\$out_id, \$out_ymd, \$out_news_flag, \$out_title, \$out_body, \$out_zeitax_publish);

			# 登録記事の一覧を表示。
			$out_data = '<div class="news_box">' . "\n" . $list_prefix . "\n";

			# DBから取り出した結果(結果集合)から１レコードずつ取り出して処理。
			while ($sth->fetch){

				# 記事の年月日を個別に保存する関数。
				my $publish_year;
				my $publish_month;
				my $publish_day;

				if($cat_src eq "tax" || $cat_src eq "biz"){
					# ゼイタックスＨＰ向けなら、ＨＰ用の日付で上書きする
					if($time_stamp eq 'zeitax'){
						$out_ymd = $out_zeitax_publish;
					};
				};

				# DB から取り出した日付データを分解する。
				if($out_ymd =~ m/(\d{4})-(\d{2})-(\d{2})/o){
					$publish_year = $1;
					$publish_month = $2;
					$publish_day = $3;
				};

				# 未来の日付になっているバックナンバーは、スキップする。
				if($jst9_year <= $publish_year && $jst9_month <= $publish_month && $jst9_day < $publish_day){
					next;
				}

				# 日付の組み立て処理
					# 年月日の表記は、クライアント側で置換処理する。
				my $publish_ymd;
				if($date_form eq "Y4MD"){
					$publish_ymd = $publish_year . '###BACKNUM_DELIMITER_YEAR###' . $publish_month . '###BACKNUM_DELIMITER_MONTH###' . $publish_day . '###BACKNUM_DELIMITER_DAY###';
				}elsif($date_form eq "Y2MD"){
					$publish_year =~ s/(\d\d)(\d\d)/$2/;
					$publish_ymd = $publish_year . '###BACKNUM_DELIMITER_YEAR###' . $publish_month . '###BACKNUM_DELIMITER_MONTH###' . $publish_day . '###BACKNUM_DELIMITER_DAY###';
				}elsif($date_form eq "MD"){
					$publish_ymd = $publish_month . '###BACKNUM_DELIMITER_MONTH###' . $publish_day . '###BACKNUM_DELIMITER_DAY###';
				}elsif($date_form eq "D"){
					$publish_ymd = $publish_day . '###BACKNUM_DELIMITER_DAY###';
				}else{
					$publish_ymd = '';
				};

				if($cat_src eq "tax" || $cat_src eq "biz"){

						if($date_form =~ m/Y?\d?M?D/o){
							if($list_style =~ m/[u?d]/io){
								$out_data .= $list_content_prefix . '<div class="info_header">' . $publish_ymd . '</div><div class="info_cage"><a href="' . $client_path . '?news_src=' . $out_id . '&cat_src=' . $cat_src . '&enc=' . $out_encode_type . '" target="' . $uri_target . '">' . $out_title . "</a></div>";
							}else{
								$out_data .= $list_content_prefix . $publish_ymd . ' ' . '<a href="' . $client_path . '?news_src=' . $out_id . '&cat_src=' . $cat_src . '&enc=' . $out_encode_type . '" target="' . $uri_target . '">' . $out_title . "</a>";
							}
						}else{
								$out_data .= $list_content_prefix . '<a href="' . $client_path . '?news_src=' . $out_id . '&cat_src=' . $cat_src . '&enc=' . $out_encode_type . '" target="' . $uri_target . '">' . $out_title . "</a>";
						};

				}elsif($cat_src eq "konew" || $cat_src eq "kotatsu" || $cat_src eq "zanew"){

						if($date_form =~ m/Y?\d?M?D/o){
							if($list_style =~ m/[u?d]/io){
								$out_data .= $list_content_prefix . '<div class="info_header">' . $publish_ymd . '</div><div class="info_cage"><a href="' . $out_body . '" target="' . $uri_target . '">' . $out_title . "</a></div>";
							}else{
								$out_data .= $list_content_prefix . $publish_ymd . ' ' . '<a href="' . $out_body . '" target="' . $uri_target . '">' . $out_title . "</a>";
							}
						}else{
								$out_data .= $list_content_prefix . '<a href="' . $out_body . '" target="' . $uri_target . '">' . $out_title . "</a>";
						};

				}

				$out_data .= $list_content_suffix . "\n";

			}; # while

			$out_data .= $list_suffix . "\n" . '</div>'; # </div>は、class="news_box" の終端。

			# 記事データをURIエンコードする。
				# 記事中の & も %26 に追加でエンコードする仕様なので注意。
			$out_data = &uri_encode($out_data);


			# タグ付けをする
				# = までURIエンコードされないように、後付けする必要がある。
			$out_data = 'backnum_body=' . $out_data;

			# フォームデータを結合する & を生かすために、必要なデータのURIエンコード後に他データを結合する
			$out_data .= '&backnum_year=' . $backnum_year;
			$out_data .= '&backnum_month=' . $backnum_month;
			$out_data .= '&news_category_name=' . $news_category_name;
			$out_data .= '&retroactive=' . $retroactive;
			$out_data .= '&backnum_axis=' . $backnum_axis;
			$out_data .= '&jst9_year=' . $jst9_year;



# ##########################################################################################
# ▼ 記事の見出しリストを出力する処理
# ##########################################################################################

		}elsif($news_src =~ m/[c]/i){

			# 見出し一覧リストのリンクについて _self か _blank を選択するスイッチを格納
			my $a_target = $form{'a_target'};

			# 見出し一覧リストのリンクについて、_self を要求されていたら、そのように変更。
			if($a_target eq 'BACK'){
				$uri_target = '_self';
			};

			# 要求された記事件数が適正かチェック
			$news_order =~ s/[^0-9]//g;

			# スクリプト実行時のタイムスタンプを作成
			my $current_timestamp = $jst9_year . "-" . $jst9_month . "-" . $jst9_day; # . " " .  $jst9_hour . ":" . $jst9_minute . ":" . $jst9_second;

			if($news_order eq ''){

				# 要求数が無い場合にはデフォルトをセット。
				$news_order = $default_subject_line;

			}elsif($news_order > $subject_line_max){

				# 要求された記事件数が最大値以上なら最大値にリセット。
				$news_order = $subject_line_max;
			}elsif($news_order < 1){

				# 記事件数が最小値以下なら最小値にリセット。
				$news_order = $subject_line_min;
			};

			my $sth;

			# ゼイタックスHP向けの日付を要求されていたら、日付の範囲指定を切り替える
			if($time_stamp eq 'zeitax'){

				$sth = $dbh->prepare("SELECT * FROM $news_category_table WHERE zeitax_publish BETWEEN ? AND ? ORDER BY zeitax_publish DESC, id DESC LIMIT ?") or die $DBI::errstr;

			}else{

				$sth = $dbh->prepare("SELECT * FROM $news_category_table WHERE publish BETWEEN ? AND ? ORDER BY publish DESCLIMIT ?") or die $DBI::errstr;

			}

			# プレースホルダーで、パラメーターをセット。
			$sth->bind_param( 1, "0" ); #過去の日付から
			$sth->bind_param( 2, $current_timestamp ); # 現在までのニュース全範囲(未来の記事を表示させない)
			$sth->bind_param( 3, $news_order ); # 取得する見出しの件数

			$sth->execute() or die $DBI::errstr;

			# DBから取得したレコードの列データ(２次元テーブル)と変数を関連づける。
			my ($out_id, $out_ymd, $out_new, $out_title, $out_body, $out_zeitax_publish);
			$sth->bind_columns(\$out_id, \$out_ymd, \$out_new ,\$out_title, \$out_body, \$out_zeitax_publish);

			# 登録記事の一覧を表示。
			$out_data = '<div class="news_box">' ."\n" . $list_prefix . "\n";

			# DBから取り出した結果(結果集合)から１レコードずつ取り出して処理。
			while ($sth->fetch){

				# 記事の年月日を個別に保存する関数。
				my $publish_year;
				my $publish_month;
				my $publish_day;

				if($cat_src eq "tax" || $cat_src eq "biz"){
					# ゼイタックスＨＰ向けなら、ＨＰ用の日付で上書きする
					if($time_stamp eq 'zeitax'){
						$out_ymd = $out_zeitax_publish;
					};
				};

				# DB から取り出した日付データを分解する。
				if($out_ymd =~ m/(\d{4})-(\d{2})-(\d{2})/){
					$publish_year = $1;
					$publish_month = $2;
					$publish_day = $3;
				};

				# 未来の日付になっているバックナンバーは、スキップする。
				if($jst9_year <= $publish_year && $jst9_month <= $publish_month && $jst9_day < $publish_day){
					next;
				}

				# 日付の組み立て処理
				# 年月日の表記は、クライアント側で置換処理する。
				my $publish_ymd;
				if($date_form eq "Y4MD"){
					$publish_ymd = $publish_year . '###LIST_DELIMITER_YEAR###' . $publish_month . '###LIST_DELIMITER_MONTH###' . $publish_day . '###LIST_DELIMITER_DAY###';
				}elsif($date_form eq "Y2MD"){
					$publish_year =~ s/(\d\d)(\d\d)/$2/;
					$publish_ymd = $publish_year . '###LIST_DELIMITER_YEAR###' . $publish_month . '###LIST_DELIMITER_MONTH###' . $publish_day . '###LIST_DELIMITER_DAY###';
				}elsif($date_form eq "MD"){
					$publish_ymd = $publish_month . '###LIST_DELIMITER_MONTH###' . $publish_day . '###LIST_DELIMITER_DAY###';
				}elsif($date_form eq "D"){
					$publish_ymd = $publish_day . '###LIST_DELIMITER_DAY###';
				}else{
					$publish_ymd = '';
				};

				if($cat_src eq "tax" || $cat_src eq "biz"){

						if($date_form =~ m/Y?\d?M?D/o){
							if($list_style =~ m/[u?d]/io){
								$out_data .= $list_content_prefix . '<div class="info_header">' . $publish_ymd . '</div><div class="info_cage"><a href="' . $client_path . '?news_src=' . $out_id . '&cat_src=' . $cat_src . '&enc=' . $out_encode_type . '" target="' . $uri_target . '">' . $out_title . "</a>";
							}else{
								$out_data .= $list_content_prefix . $publish_ymd . ' ' . '<a href="' . $client_path . '?news_src=' . $out_id . '&cat_src=' . $cat_src . '&enc=' . $out_encode_type . '" target="' . $uri_target . '">' . $out_title . "</a>";
							}
						}else{
								$out_data .= $list_content_prefix . '<a href="' . $client_path . '?news_src=' . $out_id . '&cat_src=' . $cat_src . '&enc=' . $out_encode_type . '" target="' . $uri_target . '">' . $out_title . "</a>";
						};

				}elsif($cat_src eq "konew" || $cat_src eq "kotatsu" || $cat_src eq "zanew"){

						if($date_form =~ m/Y?\d?M?D/o){
							if($list_style =~ m/[u?d]/io){
								$out_data .= $list_content_prefix . '<div class="info_header">' . $publish_ymd . '</div><div class="info_cage"><a href="' . $out_body . '" target="' . $uri_target . '">' . $out_title . "</a>";
							}else{
								$out_data .= $list_content_prefix . $publish_ymd . ' ' . '<a href="' . $out_body . '" target="' . $uri_target . '">' . $out_title . "</a>";
							}
						}else{
								$out_data .= $list_content_prefix . '<a href="' . $out_body . '" target="' . $uri_target . '">' . $out_title . "</a>";
						};

				}

				# 新着フラグ $out_new が有効なら、見出し記事の後ろに新着マークを追加。
					# pack("C", 1) についての詳細は、この if構文 の then節 を参照のこと。
				if($out_new eq pack("C", 1)){
					$out_data .= '<span class="new_flag">' . $new_mark . '</span>';
				};

				if($list_style =~ m/[u?d]/io){
					$out_data .= '</div>' . $list_content_suffix . "\n";
				}else{
					$out_data .= $list_content_suffix . "\n";
				}

			}

			$out_data .= $list_suffix . "\n" . '</div>'; # </div>は、class="news_box" の終端。


		}elsif($news_src =~ m/\d+/){



# ##########################################################################################
# ▼ 要求されたカテゴリから、記事No.の見出しと本文を抽出して出力する処理
# ##########################################################################################

		    my $state = $form{'state'};

			my($db_user, $db_pass, $db_type, $db_name, $db_locate, $db_port, $db_attr_ref) = &sqlctrl();
			my $dbh = DBI->connect("dbi:$db_type:$db_name:$db_locate:$db_port", $db_user, $db_pass, $db_attr_ref) or die $DBI::errstr;

			my $form_str;

			# サニタイズ処理。数字以外を削除。
			$news_src =~ s/\D//g;

			if(!$news_src){ die "記事が見つかりませんでした。";};

			my $sth = $dbh->prepare("SELECT * FROM $news_category_table WHERE id=?") or die $DBI::errstr;
			$sth->bind_param( 1, $news_src ); #テーブルの１列目(カラム)

			$sth->execute() or die $DBI::errstr;

			# DBから取得したレコードの列データ(２次元テーブル)と変数を関連づける。
			my ($out_id, $out_ymd, $out_news_flag, $out_title, $out_body,$out_zeitax_publish);
			$sth->bind_columns(\$out_id, \$out_ymd, \$out_news_flag, \$out_title, \$out_body, \$out_zeitax_publish);

			$sth->fetch;

			if($cat_src eq "tax" || $cat_src eq "biz"){
				# ゼイタックスＨＰ向けなら、ＨＰ用の日付で上書きする
				if($time_stamp eq 'zeitax'){
					$out_ymd = $out_zeitax_publish;
				};
			};

			# 記事の年月日を抽出する
			$out_ymd =~ m/(\d\d\d\d)-(\d\d)-(\d\d)/;

			# 記事の年月日を記録する
			my $out_year = $1;
			my $out_month = $2;
			my $out_day = $3;

			# 未来の日付になっているバックナンバーは、スキップする。
			# つまり、先行入力されている未来の記事は非表示にする。
				# 未来の日付になっているバックナンバーは、スキップする。
#			if($jst9_year <= $out_year && $jst9_month <= $out_month && $jst9_day < $out_day){
#				next;
#			}

			# 改行コードを<br />タグに置き換え
			$out_body = &crlf2br($out_body);

			# 一行ごとに<p>タグで括るようにする
			$out_body =~ s!<br /><br />!</p><p>!igm;
			$out_body =~ s!^!<p>!igm; # 改行コードはすべて<br />変換されているので、$out_body には１つしか行頭がない
			$out_body =~ s!$!</p>!igm; # 改行コードはすべて<br />変換されているので、$out_body には１つしか行末がない

			# ＵＲＬリンクの拾い上げ（おまけ機能）
			$out_body =~ s!($uri_match)!<a href="$1" target="$uri_target">$1</a>!igm;

			# メールアドレスの拾い上げ（おまけ機能）
			$out_body =~ s!($mail_match)!<a href="mailto:$1">$1</a>!igm;

			# ニュース原稿をURIエンコードする
			$out_title = uri_encode($out_title);
			$out_body = uri_encode($out_body);
			$news_category_name = &uri_encode($news_category_name);

			# ニュース情報を配信向けに編成する。
			$out_data = 'year=' . $out_year
							 . '&month=' . $out_month
							 . '&day=' . $out_day
							 . '&category=' . $news_category_name
							 . '&title=' . $out_title
							 . '&body=' . $out_body
							 . '&enc=' . $out_encode_type;

		}; # 記事出力、バックナンバー、記事見出しの判別分岐

		print $out_data;


	}else{ # ユーザーIDが合致しない場合の処理
		print "IDが登録されていない";
		print '<META HTTP-EQUIV="Refresh" CONTENT="0; URL=about:blank" />';
	}; # ユーザーIDのバリデーション

	# 接続を解除。
	$dbh->disconnect();

} # main

